using System;
using System.Windows.Forms;
using System.IO;

namespace lastporttest
{


	/// <summary>
	/// Zusammenfassung fr myRS232.
	/// </summary>
	public class myRS232 : CommBase
	{
		private CommBase.CommBaseSettings Settings;

		private byte[] buffer = new byte[4096];
		private int bufferIndex = 0;



		public myRS232()
		{
			//
			// TODO: Fgen Sie hier die Konstruktorlogik hinzu
			//
			this.Settings = new CommBase.CommBaseSettings();
			this.Settings.baudRate = 9600;
		}

		public void FillPorts(ComboBox cb)
		{
			string n;
			string angezeigterText="";
			for (int i=0; (i < 99); i++)
			{
				n = "COM" + i.ToString()+":";
				if (this.IsPortAvailable(n) == CommBase.PortStatus.available)
				{
					cb.Items.Add(n);
					if(angezeigterText==""){ angezeigterText=n; }
				}
			}
			cb.Text = angezeigterText;
		}

		public void FillBauds(ComboBox cb)
		{
			cb.Items.Clear();
			cb.Text = this.Settings.baudRate.ToString();
			cb.Items.Add("110");
			cb.Items.Add("300");
			cb.Items.Add("600");
			cb.Items.Add("1200");
			cb.Items.Add("2400");
			cb.Items.Add("4800");
			cb.Items.Add("9600");
			cb.Items.Add("19200");
			cb.Items.Add("38400");
			cb.Items.Add("57600");
			cb.Items.Add("115200");
		}

		/*musste berschrieben werden, damit open() in CommBase sich die genderten
		 * einstellungen nimmt und keine vordefinierten standart einstellungen 
		 * CommSettings() gibt normalerweise ein new ComBaseSettings zurck
		 * durchs berschreiben gibt ComSettings() jetzt aber ein bereits verndertes objekt
		 *		zurck*/
		protected override CommBaseSettings CommSettings()
		{
			return this.Settings;
		}


		protected override void OnRxChar(byte c)
		{
			//MessageBox.Show(Convert.ToString(Convert.ToChar(c)));
			if(bufferIndex<4096)
			{
				this.buffer[bufferIndex] = c;
				bufferIndex++;
			} 
			else 
			{
				throw new Exception("OnRxChar...Buffer Overflow");
			}
		}

	
	#region Write-Methoden
		public bool Write(char zeichen)
		{
			if(this.Online)
			{
				this.Send(Convert.ToByte(zeichen));
				return true;
			}
			else
			{ return false;}
		}

		public bool Write(string zeichenkette)
		{
			//funktioniert, folglich Write(char zeichen) auch!!! 08.03.2005 christoph
			if(zeichenkette.Length>0)
			{
				for(int i=0;i<zeichenkette.Length;i++)
				{
					this.Write(zeichenkette[i]);
				}
				return true;
			} 
			else
			{return false;}
		}

		public bool Write(byte zeichen)
		{
			//funktioniert, folglich Write(char zeichen) auch!!! 08.03.2005 christoph
			if(this.Online)
			{
				this.Send(zeichen);
				return true;
			}
			else
			{ return false;}
		}
		public bool Write(byte[] zeichenarray)
		{
			//funktioniert, folglich Write(char zeichen) auch!!! 08.03.2005 christoph
			if(this.Online)
			{
				this.Send(zeichenarray);
				return true;
			}
			else
			{ return false;}
		}
	#endregion
	
	#region Read-Methoden
		/// <summary>
		/// empfangene zeichen werden als string zurckgegeben und buffer wird geleert.
		/// </summary>
		/// <param name="temp">zeichen im buffer</param>
		/// <returns>true,wenn zeichen vorhanden und in parameter temp zurckgegeben werden konnten | false wenn keine zeichen in parameter temp zurckgegeben werden konnten</returns>
		public bool Read(ref string temp)
		{
			int i;
			temp = "";
			if(BufferIndex>0)
			{
				for(i=0;i<BufferIndex;i++)
				{
					temp += Convert.ToString(Convert.ToChar(this.buffer[i]));
				}
				bufferIndex=0;
				return true;
			} 
			else
			{
				return false;
			}
		}
		
/// <summary>
/// empfangene zeichen werden als byte-array zurckgegeben und buffer wird geleert.
/// </summary>
/// <param name="temp">bytes im buffer</param>
/// <returns>true, wenn bytes in temp zurckgegeben werden knnen, sonst false</returns>
		public bool Read(ref byte[] temp)
		{
			int i;
			if(BufferIndex>0)
			{
				temp=new byte[BufferIndex];
				for(i=0;i<BufferIndex;i++)
				{
					temp[i]=Buffer[i];
				}
				bufferIndex=0;
				return true;
			} 
			else 
			{
				return false;
			}
		}

		public int BytesToRead()
		{
			return BufferIndex;
		}
		
	#endregion

	#region Pin-Methoden
		public bool O_RTS
		{
			set
			{
				if(this.RTSavailable==true)
				{
					this.RTS = value;
				}
			}
			get
			{
				return this.RTS;
			}
		}

		public bool O_DTR
		{
			set
			{
				if(this.DTRavailable==true)
				{
					this.DTR = value;
				}
			}
			get
			{
				return this.DTR;
			}
		}
		public bool O_BREAK
		{
			set
			{
				if(this.Online)
				{
					this.Break = value;
				}
			}
			get
			{
				/* funktionniert scheinbar nicht richtig, jedoch an dieser stelle nicht
				 * bedeutung*/
				return this.Break;
			}
		}

	#endregion
	
	#region SetUndGet
		public byte[] Buffer
		{
			get
			{
				bufferIndex=0;
				return this.buffer;
			}
		}
		public int BufferIndex
		{
			get
			{
				return this.bufferIndex;
			}
		}
		#endregion

	#region SettingsProperties
		
		/// <summary>
		/// Format: "COM1:"
		/// </summary>
		public string ComPort
		{
			set
			{
				//
				//TO DO: format berprfen!!!
				//
				if(!this.Online)
				{
					this.Settings.port = value;
				}
				else 
				{
					MessageBox.Show("NO CHANGE WHILE PORT IS OPEN");
				}
			}

			get
			{
				return this.Settings.port;
			}
		}

		/// <summary>
		/// Baudrates (int):
		/// 110
		/// 300
		/// 600
		/// 1200
		/// 2400
		/// 4800
		/// 9600
		/// 19200
		/// 38400
		/// 57600
		/// 115200
		/// </summary>
		public int Baudrate
		{
			set
			{
				if(!this.Online)
				{
					switch(value)
					{
						case 110:
							this.Settings.baudRate = value; break;
						case 300:
							this.Settings.baudRate = value; break;
						case 600:
							this.Settings.baudRate = value; break;
						case 1200:
							this.Settings.baudRate = value; break;
						case 2400:
							this.Settings.baudRate = value; break;
						case 4800:
							this.Settings.baudRate = value; break;
						case 9600:
							this.Settings.baudRate = value; break;
						case 19200:
							this.Settings.baudRate = value; break;
						case 38400:
							this.Settings.baudRate = value; break;
						case 57600:
							this.Settings.baudRate = value; break;
						case 115200:
							this.Settings.baudRate = value; break;
						default:
							throw new Exception("Illegal Baudrate: "+value.ToString());
							break;
					} 
				}
				else 
				{
					MessageBox.Show("NO CHANGE WHILE PORT IS OPEN");
				}
			}
			get
			{
				return Settings.baudRate;
			}
		}
		#endregion

	}
}
